@@ -37,7 +37,7 @@
 static gboolean
 find_first (HTMLEngine *e)
 {
-	guchar c;
+	gunichar c;
 
 	c = html_cursor_get_current_char (e->cursor);
 	while (c == 0 || ! g_unichar_isalnum (c) || c == ' ') {
@@ -59,7 +59,8 @@ upper_lower (HTMLObject *obj, HTMLEngine *e, gpointer data)
 		old_text = HTML_TEXT (obj)->text;
 		HTML_TEXT (obj)->text = up ? g_utf8_strup (old_text, -1) : g_utf8_strdown (old_text, -1);
 		g_free (old_text);
-		HTML_TEXT (obj)->text_bytes = g_utf8_strlen (HTML_TEXT (obj)->text, -1);
+		HTML_TEXT (obj)->text_len = g_utf8_strlen (HTML_TEXT (obj)->text, -1);
+		HTML_TEXT (obj)->text_bytes = strlen (HTML_TEXT (obj)->text);
 	}
 }
 
@@ -67,8 +68,6 @@ void
 html_engine_capitalize_word (HTMLEngine *e)
 {
 	if (find_first (e)) {
-		guchar c;
-
 		html_undo_level_begin (e->undo, "Capitalize word", "Revert word capitalize");
 		html_engine_set_mark (e);
 		html_cursor_forward (e->cursor, e);
@@ -76,8 +75,7 @@ html_engine_capitalize_word (HTMLEngine *e)
 					   upper_lower, GINT_TO_POINTER (TRUE));
 		html_engine_disable_selection (e);
 
-		c = html_cursor_get_current_char (e->cursor);
-		if (g_unichar_isalnum (c)) {
+		if (g_unichar_isalnum (html_cursor_get_current_char (e->cursor))) {
 			html_engine_set_mark (e);
 			html_engine_forward_word (e);
 			html_engine_cut_and_paste (e, "down rest", "revert down rest",
@@ -108,28 +106,44 @@ set_link (HTMLObject *obj, HTMLEngine *e, gpointer data)
 	const char *complete_url = data;
 
 	if (html_object_is_text (obj) || HTML_IS_IMAGE (obj)) {
-		char *url = g_strdup (complete_url);
-		char *target = strrchr (url, '#');
+		char *url = NULL;
+		char *target = NULL;
 
-		if (target) {
-			*target = 0;
-			target ++;
+		if (complete_url) {
+			url = g_strdup (complete_url);
+			target = strrchr (url, '#');
+			if (target) {
+				*target = 0;
+				target ++;
+			}
 		}
 
 		if (html_object_is_text (obj)) {
-			html_text_add_link (HTML_TEXT (obj), e, url, target, 0, HTML_TEXT (obj)->text_len);
+			if (complete_url)
+				html_text_add_link (HTML_TEXT (obj), e, url, target, 0, HTML_TEXT (obj)->text_len);
+			else
+				html_text_remove_links (HTML_TEXT (obj));
 		} else if (HTML_IS_IMAGE (obj)) {
-			html_object_set_link (obj,
-					      url && *url
-					      ? html_colorset_get_color (e->settings->color_set, HTMLLinkColor)
-					      : html_colorset_get_color (e->settings->color_set, HTMLTextColor),
-					      url, target);
+			if (complete_url)
+				html_object_set_link (obj,
+						      url && *url
+						      ? html_colorset_get_color (e->settings->color_set, HTMLLinkColor)
+						      : html_colorset_get_color (e->settings->color_set, HTMLTextColor),
+						      url, target);
+			else
+				html_object_remove_link (obj,
+							 html_colorset_get_color (e->settings->color_set, HTMLTextColor));
 		}
+
+		g_free (url);
 	}
 }
 
 void
 html_engine_set_link (HTMLEngine *e, const char *complete_url)
 {
-	html_engine_cut_and_paste (e, "Set link", "Unset link", set_link, (gpointer) complete_url);
+	html_engine_cut_and_paste (e,
+				   complete_url ? "Set link" : "Remove link",
+				   complete_url ? "Remove link" : "Set link",
+				   set_link, (gpointer) complete_url);
 }
